

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

Represents a byte range as used in the Range header.

If @end is non-negative, then @start and @end represent the bounds
of of the range, counting from 0. (Eg, the first 500 bytes would be
represented as @start = 0 and @end = 499.)

If @end is -1 and @start is non-negative, then this represents a
range starting at @start and ending with the last byte of the
requested resource body. (Eg, all but the first 500 bytes would be
@start = 500, and @end = -1.)

If @end is -1 and @start is negative, then it represents a "suffix
range", referring to the last -@start bytes of the resource body.
(Eg, the last 500 bytes would be @start = -500 and @end = -1.)
-}

module GI.Soup.Structs.Range
    ( 

-- * Exported types
    Range(..)                               ,
    newZeroRange                            ,
    noRange                                 ,


 -- * Properties
-- ** End
    rangeReadEnd                            ,


-- ** Start
    rangeReadStart                          ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Soup.Types
import GI.Soup.Callbacks

newtype Range = Range (ForeignPtr Range)
-- | Construct a `Range` struct initialized to zero.
newZeroRange :: MonadIO m => m Range
newZeroRange = liftIO $ callocBytes 16 >>= wrapPtr Range

noRange :: Maybe Range
noRange = Nothing

rangeReadStart :: Range -> IO Int64
rangeReadStart s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO Int64
    return val

rangeReadEnd :: Range -> IO Int64
rangeReadEnd s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO Int64
    return val

type family ResolveRangeMethod (t :: Symbol) (o :: *) :: * where
    ResolveRangeMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveRangeMethod t Range, MethodInfo info Range p) => IsLabelProxy t (Range -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveRangeMethod t Range, MethodInfo info Range p) => IsLabel t (Range -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


